@@ -3,12 +3,9 @@
  *
  * Copyright (c) 2001, 2002, 2003 Jeff Horwitz (jeff@smashing.org).
  * All rights reserved.
- *
- * This package is free software; you can redistribute it and/or modify it
- * under the same terms as Perl itself.
  */
 
-/* $Id: extproc_perl.c,v 1.29 2003/04/22 01:42:27 jeff Exp $ */
+/* $Id: extproc_perl.c,v 1.36 2003/05/22 19:36:20 jeff Exp $ */
 
 #ifdef __cplusplus
 extern "C" {
@@ -27,7 +24,7 @@ extern "C" {
 }
 #endif
 
-#define EXTPROC_PERL_VERSION	"0.97"
+#define EXTPROC_PERL_VERSION	"1.0"
 
 #define PERL_NO_GET_CONTEXT
 
@@ -38,10 +35,71 @@ int _connected; /* for collaboration between extproc_perl & DBI */
 
 EXTERN_C void xs_init();
 
+#ifdef EP_DEBUGGING
+static int ep_debugging; /* debug logging flag */
+static FILE *epfp; /* debug log file structure */
+
+char *ep_debug_enable(OCIExtProcContext *ctx)
+{
+	char *fname;
+	pid_t pid;
+
+	if (ep_debugging) return;
+
+	ep_debugging = 1;
+
+	/* use oracle's memory allocation in case we're unloaded */
+	fname = OCIExtProcAllocCallMemory(ctx, MAXPATHLEN+1);
+		
+	/* open log file */
+	pid = getpid();
+	snprintf(fname, MAXPATHLEN, "/tmp/ep_debug.%d", pid);
+	if (!(epfp = fopen(fname, "a+"))) {
+		fprintf(stderr, "extproc_perl: open failed for debug log %s",
+			fname);
+		return(NULL);
+	}
+
+	/* redirect stderr */
+	dup2(fileno(epfp), fileno(stderr));
+
+	return(fname);
+}
+
+void ep_debug_disable(void)
+{
+	if (!ep_debugging) return;
+
+	ep_debugging = 0;
+
+	/* close log file & stderr */
+	fclose(epfp);
+	fclose(stderr);
+}
+
+void ep_debug(char *fmt, ...)
+{
+	va_list ap;
+	char args[32], *ts;
+	int n = 0;
+	time_t t;
+
+	va_start(ap, fmt);
+	t = time(NULL);
+	ts = ctime(&t);
+	ts[strlen(ts)-1] = '\0';
+	fprintf(epfp, "%s ", ts);
+	vfprintf(epfp, fmt, ap);
+	fprintf(epfp, "\n");
+	fflush(epfp);
+}
+#endif /* EP_DEBUGGING */
+
 void ora_exception(OCIExtProcContext *ctx, char *msg)
 {
 	char str[1024];
 
+	EP_DEBUGF("IN ora_exception(%p, \"%s\")", ctx, msg);
 	snprintf(str, 1023, "PERL EXTPROC ERROR: %s\n", msg);
 	OCIExtProcRaiseExcpWithMsg(ctx, ORACLE_USER_ERR, str, 0);
 }
@@ -50,35 +108,61 @@ PerlInterpreter *pl_startup(void)
 {
 	PerlInterpreter *p;
 	int argc;
-	char *argv[3];
+	char *argv[4];
 	struct stat st;
 	SV *sv;
 
 	dTHX;
 
+#ifdef EP_DEBUGGING
+	/* initialize ep_debug to a sane value */
+	if (ep_debugging != 1) {
+		ep_debugging = 0;
+	}
+#endif /* EP_DEBUGGING */
+	EP_DEBUG("IN pl_startup()");
+
 	/* create interpreter */
 	if((p = perl_alloc()) == NULL) {
+		EP_DEBUG("perl_alloc() failed!");
 		return(NULL);
 	}
 	PL_perl_destruct_level = 0;
 	perl_construct(p);
+	EP_DEBUGF("-- Perl interpreter created: p=%p", p);
 
 	/* parse bootstrap file if it exists */
 	if (!stat(BOOTSTRAP_FILE, &st)) {
+		EP_DEBUGF("-- Using bootstrap file '%s'", BOOTSTRAP_FILE);
 		argv[0] = "";
+#ifdef EP_TAINTING
+		argv[1] = "-T";
+		argv[2] = BOOTSTRAP_FILE;
+		argc = 3;
+#else
 		argv[1] = BOOTSTRAP_FILE;
 		argc = 2;
+#endif /* EP_TAINTING */
 	}
 	else {
+		EP_DEBUG("-- No bootstrap file found.  Move along.");
 		argv[0] = "";
+#ifdef EP_TAINTING
+		argv[1] = "-T";
+		argv[2] = "-e";
+		argv[3] = "0";
+		argc = 4;
+#else
 		argv[1] = "-e";
 		argv[2] = "0";
 		argc = 3;
+#endif /* EP_TAINTING */
 	}
 
 	if (!perl_parse(p, xs_init, argc, argv, NULL)) {
 		if (!perl_run(p)) {
 			load_module(aTHX_ PERL_LOADMOD_NOIMPORT,(SV*)newSVpv("ExtProc",0),Nullsv);
+			EP_DEBUG("-- Bootstrapping successful!");
 			return(p);
 		}
 	}
@@ -87,6 +171,7 @@ PerlInterpreter *pl_startup(void)
 
 void pl_shutdown(PerlInterpreter *p)
 {
+	EP_DEBUGF("IN pl_shutdown(%p)", p);
 	perl_destruct(p);
 	perl_free(p);
 }
@@ -97,6 +182,7 @@ static char *get_code(OCIExtProcContext *ctx, char *table, char *buf)
 	char sql[MAX_SIMPLE_QUERY_SQL];
 	int buflen = MAX_SIMPLE_QUERY_RESULT;
 
+	EP_DEBUGF("IN get_code(%p, \"%s\", %p)", ctx, table, buf);
 	snprintf(sql, MAX_SIMPLE_QUERY_SQL - 18, "select code from %s", table);
 	simple_lob_query(ctx, sql, lobl, buf, &buflen, 0);
 	return(buf);
@@ -113,27 +199,37 @@ static char *call_perl_sub(OCIExtProcContext *ctx, char *sub, char **args)
 
 	dSP;
 
+	EP_DEBUGF("IN call_perl_sub(%p, \"%s\", ...)", ctx, sub);
+
 	/* push arguments onto stack */
 	ENTER;
 	SAVETMPS;
 	PUSHMARK(SP);
 	p = args;
 	while (*p) {
-		XPUSHs(sv_2mortal(newSVpv(*p++,0)));
+		sv = sv_2mortal(newSVpv(*p++,0));
+#ifdef EP_TAINTING
+		SvTAINTED_on(sv);
+#endif EP_TAINTING
+		XPUSHs(sv);
 	}
 	PUTBACK;
 
 	/* run subroutine */
+	EP_DEBUG("-- about to call call_pv()");
 	nret = call_pv(sub, G_SCALAR|G_EVAL);
+	EP_DEBUGF("-- call_pv() returned %d", nret);
 	SPAGAIN;
 
 	/* grab return value, detecting errors along the way */
 	if (SvTRUE(ERRSV) || nret != 1) {
+		EP_DEBUGF("-- ERRSV is defined: %s", SvPV(ERRSV, PL_na));
 		ora_exception(ctx, SvPV(ERRSV, PL_na));
 		POPs;
 		retval = NULL;
 	}
 	else {
+		EP_DEBUG("-- No errors detected");
 		sv = POPs;
 		tmp = SvPV(sv,len);
 		/* use oracle's memory allocation in case we're unloaded */
@@ -156,9 +252,12 @@ char *ora_perl_func(OCIExtProcContext *ctx, OCIInd *ret_ind, char *sub, ...)
 	va_list ap;
 	short ind;
 	char *args[MAXARGS], *retval, code[MAX_CODE_SIZE], *errbuf;
+	SV *evalsv;
 
 	dTHX;
 
+	EP_DEBUGF("IN ora_perl_func(%p, %p, \"%s\", ...)", ctx, ret_ind, sub);
+
 	/* set OCI context for ExtProc module */
 	this_ctx.ctx = ctx;
 
@@ -179,6 +278,27 @@ char *ora_perl_func(OCIExtProcContext *ctx, OCIInd *ret_ind, char *sub, ...)
 	}
 	va_end(ap);
 
+	EP_DEBUGF("-- found %d argument(s)", n);
+
+#ifdef EP_DEBUGGING
+	/* check for debug request */
+	if (!strncmp(sub, "_enable_debug", 13)) {
+		if ((retval = ep_debug_enable(ctx))) {
+			*ret_ind = OCI_IND_NOTNULL;
+		}
+		else {
+			*ret_ind = OCI_IND_NULL;
+		}
+		return(retval);
+	}
+
+	if (!strncmp(sub, "_disable_debug", 14)) {
+		ep_debug_disable();
+		*ret_ind = OCI_IND_NULL;
+		return("\0");
+	}
+#endif /* DEBUGGING */
+
 	/* check for flush request */
 	if (!strncmp(sub, "_flush", 6)) {
 		/* only destroy the interpreter if it exists */
@@ -261,12 +381,13 @@ char *ora_perl_func(OCIExtProcContext *ctx, OCIInd *ret_ind, char *sub, ...)
 
 	/* check for eval request */
 	if (!strncmp(sub, "_eval", 6)) {
-		eval_pv(args[0], FALSE);
+		evalsv = eval_pv(args[0], FALSE);
 		if (SvTRUE(ERRSV)) {
 			ora_exception(ctx, SvPV(ERRSV, PL_na));
 		}
-		*ret_ind = OCI_IND_NULL;
-		return("\0");
+		*ret_ind = SvOK(evalsv) ? OCI_IND_NOTNULL : OCI_IND_NULL;
+		retval = SvPV(sv_2mortal(evalsv), PL_na);
+		return(retval);
 	}
 
 	/*
@@ -274,20 +395,27 @@ char *ora_perl_func(OCIExtProcContext *ctx, OCIInd *ret_ind, char *sub, ...)
 	 * try loading code from database if it isn't initially valid.
 	 */
 	if (!get_cv(sub, FALSE)) {
+		EP_DEBUG("-- CV does not exist, trying database...");
 		/* load code -- fail silently if no code is available */
 		get_code(ctx, code_table, code);
 
 		/* parse code */
+		EP_DEBUG("-- parsing code from database...");
 		eval_pv(code, TRUE);
+		EP_DEBUG("-- parsing successful");
 
 		/* try again */
 		if (!get_cv(sub, FALSE)) {
+			EP_DEBUG("-- CV STILL doesn't exist!");
 			*ret_ind = OCI_IND_NULL;
 			ora_exception(ctx, "invalid subroutine");
 			return("\0");
 		}
+		EP_DEBUG("-- CV exists!");
 	}
 
+	EP_DEBUG("-- CV is cached");
+
 	/* run subroutine */
 	retval = call_perl_sub(ctx, sub, args);
 	*ret_ind = retval ? OCI_IND_NOTNULL : OCI_IND_NULL;
@@ -305,6 +433,8 @@ void ora_perl_proc(OCIExtProcContext *ctx, char *sub, ...)
 
 	dTHX;
 
+	EP_DEBUGF("IN ora_perl_proc(%p, \"%s\", ...)", ctx, sub);
+
 	/* set OCI context for ExtProc module */
 	this_ctx.ctx = ctx;
 
@@ -325,6 +455,21 @@ void ora_perl_proc(OCIExtProcContext *ctx, char *sub, ...)
 	}
 	va_end(ap);
 
+	EP_DEBUGF("-- found %d argument(s)", n);
+
+#ifdef EP_DEBUGGING
+	/* check for debug request */
+	if (!strncmp(sub, "_enable_debug", 13)) {
+		ep_debug_enable(ctx);
+		return;
+	}
+
+	if (!strncmp(sub, "_disable_debug", 14)) {
+		ep_debug_disable();
+		return;
+	}
+#endif /* EP_DEBUGGING */
+
 	/* check for flush request */
 	if (!strncmp(sub, "_flush", 6)) {
 		/* only destroy the interpreter if it exists */
