@@ -1,14 +1,11 @@
 /*
  * Oracle Perl Procedure Library
  *
- * Copyright (c) 2001, 2002 Jeff Horwitz (jeff@smashing.org).
+ * Copyright (c) 2001, 2002, 2003 Jeff Horwitz (jeff@smashing.org).
  * All rights reserved.
- *
- * This package is free software; you can redistribute it and/or modify it
- * under the same terms as Perl itself.
  */
 
-/* $Id: extproc_perl.h,v 1.14 2003/04/20 20:11:53 jeff Exp $ */
+/* $Id: extproc_perl.h,v 1.17 2003/05/22 16:37:51 jeff Exp $ */
 
 #ifndef EXTPROC_PERL_H
 #define EXTPROC_PERL_H
@@ -19,6 +16,14 @@
 #define MAX_SIMPLE_QUERY_SQL	256
 #define MAX_CODE_SIZE		8192 /* <= MAX_SIMPLE_QUERY_RESULT */
 
+#ifdef EP_DEBUGGING
+#define EP_DEBUG(msg) if (ep_debugging) { ep_debug("%s", msg); }
+#define EP_DEBUGF(fmt, ...) if (ep_debugging) { ep_debug(fmt, __VA_ARGS__); }
+#else
+#define EP_DEBUG(msg) 1;
+#define EP_DEBUGF(fmt, ...) 1;
+#endif /* EP_DEBUGGING */
+
 struct ocictx {
 	OCIExtProcContext *ctx;	/* For OCI ExtProc Context */
 	OCIEnv *envhp;		/* For OCI Environment Handle */
